package jalview.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jalview.io.EBIFetchClient;
import MCview.*;
import jalview.datamodel.*;
import jalview.analysis.AlignSeq;
import java.io.File;
import jalview.io.*;
import java.util.*;


public class SequenceFetcher
    extends JPanel implements Runnable
{
  JInternalFrame frame;
  AlignFrame alignFrame;
  StringBuffer result;
  public SequenceFetcher(AlignFrame af)
  {
    alignFrame = af;
    database.addItem("-- Select Database --");
    database.addItem("Uniprot");
    database.addItem("EMBL");
    database.addItem("EMBLCDS");
    database.addItem("PDB");

    try
    {
      jbInit();
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }

    frame = new JInternalFrame();
    frame.setContentPane(this);
    if(System.getProperty("os.name").startsWith("Mac"))
      Desktop.addInternalFrame(frame, "Sequence Fetcher (WSDBfetch@EBI)", 400, 140);
    else
      Desktop.addInternalFrame(frame, "Sequence Fetcher (WSDBfetch@EBI)", 400, 120);
  }

  private void jbInit()
      throws Exception
  {
    this.setLayout(gridBagLayout1);

    database.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    database.setMinimumSize(new Dimension(160, 21));
    database.setPreferredSize(new Dimension(160, 21));
    jLabel1.setFont(new java.awt.Font("Verdana", Font.ITALIC, 11));
    jLabel1.setText(
        "Separate multiple accession ids with semi colon \";\"");
    ok.setText("OK");
    ok.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ok_actionPerformed(e);
      }
    });
    cancel.setText("Cancel");
    cancel.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cancel_actionPerformed(e);
      }
    });
    textfield.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    textfield.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ok_actionPerformed(e);
      }
    });
    jPanel1.add(ok);
    jPanel1.add(cancel);
    this.add(jLabel1, new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
                                             , GridBagConstraints.WEST,
                                             GridBagConstraints.NONE,
                                             new Insets(7, 4, 0, 6), 77, 6));
    this.add(jPanel1, new GridBagConstraints(0, 2, 2, 1, 1.0, 1.0
                                             , GridBagConstraints.WEST,
                                             GridBagConstraints.BOTH,
                                             new Insets(7, -2, 7, 12), 241, -2));
    this.add(database, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
                                              , GridBagConstraints.WEST,
                                              GridBagConstraints.NONE,
                                              new Insets(0, 4, 0, 0), 1, 0));
    this.add(textfield, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
                                               , GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(0, 0, 0, 6), 200, 1));
  }

  JComboBox database = new JComboBox();
  JLabel jLabel1 = new JLabel();
  JButton ok = new JButton();
  JButton cancel = new JButton();
  JPanel jPanel1 = new JPanel();
  JTextField textfield = new JTextField();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  public void cancel_actionPerformed(ActionEvent e)
  {
    try
    {
      frame.setClosed(true);
    }
    catch (Exception ex)
    {}
  }

  public void ok_actionPerformed(ActionEvent e)
  {
    frame.setTitle("Sequence Fetcher (WSDBfetch@EBI) - Fetching Sequence...");
    database.setEnabled(false);
    textfield.setEnabled(false);
    ok.setEnabled(false);
    cancel.setEnabled(false);

    Thread worker = new Thread(this);
    worker.start();
  }

  public void run()
  {
   result = new StringBuffer();
    if (database.getSelectedItem().equals("Uniprot"))
    {
      getUniprotFile(textfield.getText());
    }
    else if (database.getSelectedItem().equals("EMBL")
        || database.getSelectedItem().equals("EMBLCDS"))
    {
      EBIFetchClient dbFetch = new EBIFetchClient();
      String[] reply = dbFetch.fetchData(
          database.getSelectedItem().toString().toLowerCase(
          ) + ":" + textfield.getText(),
          "fasta", "raw");

      for (int i = 0; i < reply.length; i++)
        result.append(reply[i] + "\n");

      parseResult(result.toString());
    }
    else if (database.getSelectedItem().equals("PDB"))
    {
      result = getPDBFile(textfield.getText().toUpperCase());
      parseResult(result.toString());
    }


    if (result == null || result.length() == 0)
      showErrorMessage("Error retrieving " + textfield.getText()
                       + " from " + database.getSelectedItem());


      database.setEnabled(true);
      textfield.setEnabled(true);
      ok.setEnabled(true);
      cancel.setEnabled(true);
      frame.setTitle("Sequence Fetcher (WSDBfetch@EBI)");
  }

  void getUniprotFile(String id)
  {
    EBIFetchClient ebi = new EBIFetchClient();
    File file = ebi.fetchDataAsFile("uniprot:"+id,"xml", null);
    SequenceFeatureFetcher sff = new SequenceFeatureFetcher();
    Vector entries = sff.getUniprotEntries(file);

    if(entries!=null)
    {
      //First, make the new sequences
      Enumeration en = entries.elements();
      while (en.hasMoreElements())
      {
        UniprotEntry entry = (UniprotEntry) en.nextElement();
        StringBuffer name = new StringBuffer( ">Uniprot/Swiss-Prot" );
        Enumeration en2 = entry.getAccession().elements();
        while(en2.hasMoreElements())
        {
          name.append("|");
          name.append(en2.nextElement());
        }
        en2 = entry.getName().elements();
        while(en2.hasMoreElements())
        {
          name.append("|");
          name.append(en2.nextElement());
        }

        if(entry.getProteinName()!=null)
          name.append(" "+entry.getProteinName().elementAt(0));

        result.append(name +"\n"+entry.getUniprotSequence().getContent());

      }

      //Then read in the features and apply them to the dataset
      SequenceI [] sequence = parseResult(result.toString());
      for(int i=0; i<entries.size(); i++)
      {
        UniprotEntry entry = (UniprotEntry) entries.elementAt(i);
        Enumeration e = entry.getDbReference().elements();
        Vector onlyPdbEntries = new Vector();
        while (e.hasMoreElements())
        {
          PDBEntry pdb = (PDBEntry) e.nextElement();
          if (!pdb.getType().equals("PDB"))
            continue;

          onlyPdbEntries.addElement(pdb);
        }

        sequence[i].getDatasetSequence().setPDBId(onlyPdbEntries);
        sequence[i].getDatasetSequence().setSequenceFeatures(entry.getFeature());

      }
    }
  }

    StringBuffer getPDBFile(String id)
    {
      StringBuffer result = new StringBuffer();
      String chain = null;
      if (id.indexOf(":") > -1)
      {
        chain = id.substring(id.indexOf(":") + 1);
        id = id.substring(0, id.indexOf(":"));
      }

      EBIFetchClient ebi = new EBIFetchClient();
      String[] reply = ebi.fetchData("pdb:" + id, "pdb", "raw");
      try
      {
        PDBfile pdbfile = new PDBfile(reply);
        for (int i = 0; i < pdbfile.chains.size(); i++)
        {
          if (chain == null ||
              ( (PDBChain) pdbfile.chains.elementAt(i)).id.
              toUpperCase().equals(chain))
            result.append("\n>PDB|" + id + "|" +
                          ( (PDBChain) pdbfile.chains.elementAt(i)).sequence.
                          getName() +
                          "\n"
                          +
                          ( (PDBChain) pdbfile.chains.elementAt(i)).sequence.
                          getSequence());
        }
      }
      catch (Exception ex) // Problem parsing PDB file
      {
        showErrorMessage("Error retrieving " + textfield.getText() + " from " +
            database.getSelectedItem());
        return null;
      }

      return result;
    }

    SequenceI[] parseResult(String result)
    {
      String format = IdentifyFile.Identify(result, "Paste");
      SequenceI[] sequences = null;

      if (FormatAdapter.formats.contains(format))
      {
        sequences = new FormatAdapter().readFile(result.toString(), "Paste", format);
        if (sequences != null && sequences.length>0)
        {
          if(alignFrame==null)
          {
            AlignFrame af = new AlignFrame(new Alignment(sequences));
            af.currentFileFormat = format;
            Desktop.addInternalFrame(af, "Retrieved from "+database.getSelectedItem(),
                                     AlignFrame.NEW_WINDOW_WIDTH,
                                     AlignFrame.NEW_WINDOW_HEIGHT);
            af.statusBar.setText("Successfully pasted alignment file");
            try
            {
              af.setMaximum(jalview.bin.Cache.getDefault("SHOW_FULLSCREEN", false));
            }
            catch (Exception ex)
            {}
          }
          else
          {
            for (int i = 0; i < sequences.length; i++)
            {
              alignFrame.viewport.alignment.addSequence(sequences[i]);

              ////////////////////////////
              //Datset needs extension;
              /////////////////////////////
              Sequence ds = new Sequence(sequences[i].getName(),
                                         AlignSeq.extractGaps("-. ",
                  sequences[i].getSequence()),
                                         sequences[i].getStart(),
                                         sequences[i].getEnd());
              sequences[i].setDatasetSequence(ds);
              alignFrame.viewport.alignment.getDataset().addSequence(ds);
            }
            alignFrame.viewport.setEndSeq(alignFrame.viewport.alignment.
                                          getHeight());
            alignFrame.viewport.alignment.getWidth();
            alignFrame.viewport.firePropertyChange("alignment", null,
            alignFrame.viewport.getAlignment().getSequences());

          }

          if (database.getSelectedItem().equals("PDB"))
          {
            for (int i = 0; i < sequences.length; i++)
            {
              PDBEntry entry = new PDBEntry();
              entry.setId(textfield.getText());
              sequences[i].getDatasetSequence().addPDBId(entry);
            }
          }

        }
        else
          showErrorMessage( "Error retrieving "+textfield.getText()
                            +" from "+database.getSelectedItem());
      }

      return sequences;

  }

  void showErrorMessage(String error)
  {
    JOptionPane.showInternalMessageDialog(Desktop.desktop,
         error, "Error Retrieving Data", JOptionPane.WARNING_MESSAGE);
  }
}




